"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.lowerCaseFileNames = void 0;
const LintRuleType_1 = require("../../types/LintRuleType");
const Severity_1 = require("../../types/Severity");
const path_1 = __importDefault(require("path"));
const name = 'lowerCaseFileNames';
const description = 'Enforce the use of lower case file names.';
const message = 'File name contains uppercase characters';
const test = (value, config) => {
    const severity = (config === null || config === void 0 ? void 0 : config.severityLevel[name]) || Severity_1.Severity.Warning;
    const fileName = path_1.default.basename(value);
    if (fileName.toLocaleLowerCase() === fileName)
        return [];
    return [
        {
            message,
            lineNumber: 1,
            startColumnNumber: 1,
            endColumnNumber: 1,
            severity
        }
    ];
};
/**
 * Lint rule that checks for the absence of uppercase characters in a given file name.
 */
exports.lowerCaseFileNames = {
    type: LintRuleType_1.LintRuleType.Path,
    name,
    description,
    message,
    test
};
//# sourceMappingURL=lowerCaseFileNames.js.map