"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.noSpacesInFileNames = void 0;
const LintRuleType_1 = require("../../types/LintRuleType");
const Severity_1 = require("../../types/Severity");
const path_1 = __importDefault(require("path"));
const name = 'noSpacesInFileNames';
const description = 'Enforce the absence of spaces within file names.';
const message = 'File name contains spaces';
const test = (value, config) => {
    const severity = (config === null || config === void 0 ? void 0 : config.severityLevel[name]) || Severity_1.Severity.Warning;
    const fileName = path_1.default.basename(value);
    if (fileName.includes(' ')) {
        return [
            {
                message,
                lineNumber: 1,
                startColumnNumber: 1,
                endColumnNumber: 1,
                severity
            }
        ];
    }
    return [];
};
/**
 * Lint rule that checks for the absence of spaces in a given file name.
 */
exports.noSpacesInFileNames = {
    type: LintRuleType_1.LintRuleType.Path,
    name,
    description,
    message,
    test
};
//# sourceMappingURL=noSpacesInFileNames.js.map